;
;	USB Loader/Saver API for PC-6001mkII ver.0.1
;	by Koichi Nishida 2007	
;

PRSTR	EQU	30CFH
STPESC	EQU	274DH

; commands
LDDAT	EQU	1	; load data
SVDAT	EQU	2	; save data

OK	EQU	1
NG	EQU	2

; <---------- begin example ----------
;
;	ORG	0C000H
;
;	; test program
;
;	; load data
;	LD	HL,0D000H	; start
;	LD	BC,100H		; length
;	LD	DE,FNM1		; file name
;	CALL	LDDATA
;
;	; save data
;	LD	HL,0D000H	; start
;	LD	BC,100H		; length
;	LD	DE,FNM2		; file name
;	CALL	SVDATA
;
;	RET
;
;FNM1:	DB	'DATA'
;	DB	0, 0, 0
;
;FNM2:	DB	'DATA2'
;	DB	0, 0
;
; ---------- end example ---------->

	; load data
	; HL = begin address
	; BC = length
	; DE = file name
LDDATA:
	; output LDDAT command
	CALL	WRINI
	LD	A,LDDAT
	CALL	WRITE

	; send file name
	CALL	WRFILE

	; send length
	LD	A,C
	CALL	WRITE
	LD	A,B
	CALL	WRITE

	CALL	RDINI

	; reaction
	CALL	READ
	CP	OK
	JP	NZ,ERROR	
LDLP:
	CALL	READ
	LD	(HL),A
	INC	HL
	DEC	BC
	LD	A,B
	OR	C
	JR	NZ,LDLP
	RET

	; save data
	; HL = begin address
	; BC = length
	; DE = file name
SVDATA:
	; output SVDAT command
	CALL	WRINI
	LD	A,SVDAT
	CALL	WRITE

	; send file name
	CALL	WRFILE

	; send length
	LD	A,C
	CALL	WRITE
	LD	A,B
	CALL	WRITE

	CALL	RDINI

	; reaction
	CALL	READ
	CP	OK
	JP	NZ,ERROR	

	CALL	WRINI
SVLP:
	LD	A,(HL)
	CALL	WRITE
	INC	HL
	DEC	BC
	LD	A,B
	OR	C
	JR	NZ,SVLP
	RET
ERROR:
	LD	HL,ERRSTR
	JP	PRSTR

	; stop key check
CHKSTP:	LD	A,(STPCTR)
	INC	A
	LD	(STPCTR),A
	RET	NZ
	CALL	STPESC
	RET

	; initialize 8255
RDINI:	LD	A,93H
	OUT	(0D3H),A
	RET
WRINI:	LD	A,83H
	OUT	(0D3H),A
	RET

	; write one byte
WRITE:	PUSH	AF
WRLP1:	CALL	CHKSTP
	IN	A,(0D2H)
	AND	02H
	JR	NZ,WRLP1
	POP	AF
	OUT	(0D0H),A
	LD	A,0BH
	OUT	(0D3H),A
	LD	A,0AH
	OUT	(0D3H),A
	RET
	; read one byte
READ:
RDLP:
	CALL	CHKSTP
	IN	A,(0D2H)
	AND	01H
	JR	NZ,RDLP
	LD	A,08H
	OUT	(0D3H),A
	IN	A,(0D0H)
	PUSH	AF
	LD	A,09H
	OUT	(0D3H),A
	POP	AF
	RET

	; write file name
WRFILE: 
	PUSH	BC
	LD	B,7
WRFLLP: LD	A,(DE)
	CALL	WRITE
	INC	DE
	DJNZ	WRFLLP
	POP	BC
	RET

	; for stop key check
STPCTR:	DB	0

	; messages
ERRSTR:	DB	'ERROR'
	DB	0DH,0AH,0
